@@ -227,6 +227,7 @@ class Page(ChannelOwner):
             ),
         )
         self._channel.on("video", lambda params: self._on_video(params))
+        self._channel.on("viewportSizeChanged", self._on_viewport_size_changed)
         self._channel.on(
             "webSocket",
             lambda params: self.emit(
@@ -286,7 +287,7 @@ class Page(ChannelOwner):
         route_handlers = self._routes.copy()
         for route_handler in route_handlers:
             # If the page was closed we stall all requests right away.
-            if self._close_was_called or self.context._close_was_called:
+            if self._close_was_called or self.context._closing_or_closed:
                 return
             if not route_handler.matches(route.request.url):
                 continue
@@ -363,6 +364,9 @@ class Page(ChannelOwner):
         artifact = from_channel(params["artifact"])
         self._force_video()._artifact_ready(artifact)
 
+    def _on_viewport_size_changed(self, params: Any) -> None:
+        self._viewport_size = params["viewportSize"]
+
     @property
     def context(self) -> "BrowserContext":
         return self._browser_context
@@ -384,9 +388,7 @@ class Page(ChannelOwner):
         for frame in self._frames:
             if name and frame.name == name:
                 return frame
-            if url and url_matches(
-                self._browser_context._options.get("baseURL"), frame.url, url
-            ):
+            if url and url_matches(self._browser_context._base_url, frame.url, url):
                 return frame
 
         return None
@@ -397,13 +399,9 @@ class Page(ChannelOwner):
 
     def set_default_navigation_timeout(self, timeout: float) -> None:
         self._timeout_settings.set_default_navigation_timeout(timeout)
-        self._channel.send_no_reply(
-            "setDefaultNavigationTimeoutNoReply", dict(timeout=timeout)
-        )
 
     def set_default_timeout(self, timeout: float) -> None:
         self._timeout_settings.set_default_timeout(timeout)
-        self._channel.send_no_reply("setDefaultTimeoutNoReply", dict(timeout=timeout))
 
     async def query_selector(
         self,
@@ -447,12 +445,14 @@ class Page(ChannelOwner):
     async def is_hidden(
         self, selector: str, strict: bool = None, timeout: float = None
     ) -> bool:
-        return await self._main_frame.is_hidden(**locals_to_params(locals()))
+        # timeout is deprecated and does nothing
+        return await self._main_frame.is_hidden(selector=selector, strict=strict)
 
     async def is_visible(
         self, selector: str, strict: bool = None, timeout: float = None
     ) -> bool:
-        return await self._main_frame.is_visible(**locals_to_params(locals()))
+        # timeout is deprecated and does nothing
+        return await self._main_frame.is_visible(selector=selector, strict=strict)
 
     async def dispatch_event(
         self,
@@ -519,12 +519,16 @@ class Page(ChannelOwner):
             )
         self._bindings[name] = callback
         await self._channel.send(
-            "exposeBinding", dict(name=name, needsHandle=handle or False)
+            "exposeBinding",
+            None,
+            dict(name=name, needsHandle=handle or False),
         )
 
     async def set_extra_http_headers(self, headers: Dict[str, str]) -> None:
         await self._channel.send(
-            "setExtraHTTPHeaders", dict(headers=serialize_headers(headers))
+            "setExtraHTTPHeaders",
+            None,
+            dict(headers=serialize_headers(headers)),
         )
 
     @property
@@ -557,7 +561,11 @@ class Page(ChannelOwner):
         waitUntil: DocumentLoadState = None,
     ) -> Optional[Response]:
         return from_nullable_channel(
-            await self._channel.send("reload", locals_to_params(locals()))
+            await self._channel.send(
+                "reload",
+                self._timeout_settings.navigation_timeout,
+                locals_to_params(locals()),
+            )
         )
 
     async def wait_for_load_state(
@@ -588,7 +596,11 @@ class Page(ChannelOwner):
         waitUntil: DocumentLoadState = None,
     ) -> Optional[Response]:
         return from_nullable_channel(
-            await self._channel.send("goBack", locals_to_params(locals()))
+            await self._channel.send(
+                "goBack",
+                self._timeout_settings.navigation_timeout,
+                locals_to_params(locals()),
+            )
         )
 
     async def go_forward(
@@ -597,11 +609,15 @@ class Page(ChannelOwner):
         waitUntil: DocumentLoadState = None,
     ) -> Optional[Response]:
         return from_nullable_channel(
-            await self._channel.send("goForward", locals_to_params(locals()))
+            await self._channel.send(
+                "goForward",
+                self._timeout_settings.navigation_timeout,
+                locals_to_params(locals()),
+            )
         )
 
     async def request_gc(self) -> None:
-        await self._channel.send("requestGC")
+        await self._channel.send("requestGC", None)
 
     async def emulate_media(
         self,
@@ -630,18 +646,22 @@ class Page(ChannelOwner):
             params["contrast"] = (
                 "no-override" if params["contrast"] == "null" else contrast
             )
-        await self._channel.send("emulateMedia", params)
+        await self._channel.send("emulateMedia", None, params)
 
     async def set_viewport_size(self, viewportSize: ViewportSize) -> None:
         self._viewport_size = viewportSize
-        await self._channel.send("setViewportSize", locals_to_params(locals()))
+        await self._channel.send(
+            "setViewportSize",
+            None,
+            locals_to_params(locals()),
+        )
 
     @property
     def viewport_size(self) -> Optional[ViewportSize]:
         return self._viewport_size
 
     async def bring_to_front(self) -> None:
-        await self._channel.send("bringToFront")
+        await self._channel.send("bringToFront", None)
 
     async def add_init_script(
         self, script: str = None, path: Union[str, Path] = None
@@ -652,7 +672,7 @@ class Page(ChannelOwner):
             )
         if not isinstance(script, str):
             raise Error("Either path or script parameter must be specified")
-        await self._channel.send("addInitScript", dict(source=script))
+        await self._channel.send("addInitScript", None, dict(source=script))
 
     async def route(
         self, url: URLMatch, handler: RouteHandlerCallback, times: int = None
@@ -660,7 +680,7 @@ class Page(ChannelOwner):
         self._routes.insert(
             0,
             RouteHandler(
-                self._browser_context._options.get("baseURL"),
+                self._browser_context._base_url,
                 url,
                 handler,
                 True if self._dispatcher_fiber else False,
@@ -688,24 +708,21 @@ class Page(ChannelOwner):
         behavior: Literal["default", "ignoreErrors", "wait"] = None,
     ) -> None:
         self._routes = remaining
-        await self._update_interception_patterns()
-        if behavior is None or behavior == "default":
-            return
-        await asyncio.gather(
-            *map(
-                lambda route: route.stop(behavior),  # type: ignore
-                removed,
+        if behavior is not None and behavior != "default":
+            await asyncio.gather(
+                *map(
+                    lambda route: route.stop(behavior),  # type: ignore
+                    removed,
+                )
             )
-        )
+        await self._update_interception_patterns()
 
     async def route_web_socket(
         self, url: URLMatch, handler: WebSocketRouteHandlerCallback
     ) -> None:
         self._web_socket_routes.insert(
             0,
-            WebSocketRouteHandler(
-                self._browser_context._options.get("baseURL"), url, handler
-            ),
+            WebSocketRouteHandler(self._browser_context._base_url, url, handler),
         )
         await self._update_web_socket_interception_patterns()
 
@@ -750,7 +767,9 @@ class Page(ChannelOwner):
     async def _update_interception_patterns(self) -> None:
         patterns = RouteHandler.prepare_interception_patterns(self._routes)
         await self._channel.send(
-            "setNetworkInterceptionPatterns", {"patterns": patterns}
+            "setNetworkInterceptionPatterns",
+            None,
+            {"patterns": patterns},
         )
 
     async def _update_web_socket_interception_patterns(self) -> None:
@@ -758,7 +777,9 @@ class Page(ChannelOwner):
             self._web_socket_routes
         )
         await self._channel.send(
-            "setWebSocketInterceptionPatterns", {"patterns": patterns}
+            "setWebSocketInterceptionPatterns",
+            None,
+            {"patterns": patterns},
         )
 
     async def screenshot(
@@ -792,7 +813,9 @@ class Page(ChannelOwner):
                     params["mask"],
                 )
             )
-        encoded_binary = await self._channel.send("screenshot", params)
+        encoded_binary = await self._channel.send(
+            "screenshot", self._timeout_settings.timeout, params
+        )
         decoded_binary = base64.b64decode(encoded_binary)
         if path:
             make_dirs_for_file(path)
@@ -806,7 +829,7 @@ class Page(ChannelOwner):
         self._close_reason = reason
         self._close_was_called = True
         try:
-            await self._channel.send("close", locals_to_params(locals()))
+            await self._channel.send("close", None, locals_to_params(locals()))
             if self._owned_context:
                 await self._owned_context.close()
         except Exception as e:
@@ -1105,7 +1128,9 @@ class Page(ChannelOwner):
         try:
             await asyncio.wait(
                 [
-                    asyncio.create_task(self._browser_context._channel.send("pause")),
+                    asyncio.create_task(
+                        self._browser_context._channel.send("pause", None)
+                    ),
                     self._closed_or_crashed_future,
                 ],
                 return_when=asyncio.FIRST_COMPLETED,
@@ -1137,7 +1162,7 @@ class Page(ChannelOwner):
         params = locals_to_params(locals())
         if "path" in params:
             del params["path"]
-        encoded_binary = await self._channel.send("pdf", params)
+        encoded_binary = await self._channel.send("pdf", None, params)
         decoded_binary = base64.b64decode(encoded_binary)
         if path:
             make_dirs_for_file(path)
@@ -1156,7 +1181,7 @@ class Page(ChannelOwner):
         # Note: we are creating Video object lazily, because we do not know
         # BrowserContextOptions when constructing the page - it is assigned
         # too late during launchPersistentContext.
-        if not self._browser_context._options.get("recordVideo"):
+        if not self._browser_context._videos_dir:
             return None
         return self._force_video()
 
@@ -1243,7 +1268,7 @@ class Page(ChannelOwner):
         def my_predicate(request: Request) -> bool:
             if not callable(urlOrPredicate):
                 return url_matches(
-                    self._browser_context._options.get("baseURL"),
+                    self._browser_context._base_url,
                     request.url,
                     urlOrPredicate,
                 )
@@ -1275,7 +1300,7 @@ class Page(ChannelOwner):
         def my_predicate(request: Response) -> bool:
             if not callable(urlOrPredicate):
                 return url_matches(
-                    self._browser_context._options.get("baseURL"),
+                    self._browser_context._base_url,
                     request.url,
                     urlOrPredicate,
                 )
@@ -1347,6 +1372,7 @@ class Page(ChannelOwner):
             return
         uid = await self._channel.send(
             "registerLocatorHandler",
+            None,
             {
                 "selector": locator._selector,
                 "noWaitAfter": noWaitAfter,
@@ -1387,7 +1413,9 @@ class Page(ChannelOwner):
             try:
                 await self._connection.wrap_api_call(
                     lambda: self._channel.send(
-                        "resolveLocatorHandlerNoReply", {"uid": uid, "remove": remove}
+                        "resolveLocatorHandlerNoReply",
+                        None,
+                        {"uid": uid, "remove": remove},
                     ),
                     is_internal=True,
                 )
@@ -1398,7 +1426,11 @@ class Page(ChannelOwner):
         for uid, data in self._locator_handlers.copy().items():
             if data.locator._equals(locator):
                 del self._locator_handlers[uid]
-                self._channel.send_no_reply("unregisterLocatorHandler", {"uid": uid})
+                self._channel.send_no_reply(
+                    "unregisterLocatorHandler",
+                    None,
+                    {"uid": uid},
+                )
 
 
 class Worker(ChannelOwner):
@@ -1430,6 +1462,7 @@ class Worker(ChannelOwner):
         return parse_result(
             await self._channel.send(
                 "evaluateExpression",
+                None,
                 dict(
                     expression=expression,
                     arg=serialize_argument(arg),
@@ -1443,6 +1476,7 @@ class Worker(ChannelOwner):
         return from_channel(
             await self._channel.send(
                 "evaluateExpressionHandle",
+                None,
                 dict(
                     expression=expression,
                     arg=serialize_argument(arg),
@@ -1468,12 +1502,14 @@ class BindingCall(ChannelOwner):
                 result = func(source, *func_args)
             if inspect.iscoroutine(result):
                 result = await result
-            await self._channel.send("resolve", dict(result=serialize_argument(result)))
+            await self._channel.send(
+                "resolve", None, dict(result=serialize_argument(result))
+            )
         except Exception as e:
             tb = sys.exc_info()[2]
             asyncio.create_task(
                 self._channel.send(
-                    "reject", dict(error=dict(error=serialize_error(e, tb)))
+                    "reject", None, dict(error=dict(error=serialize_error(e, tb)))
                 )
             )
 
