@@ -66,7 +66,7 @@ class CRBrowser extends import_browser.Browser {
   }
   static async connect(parent, transport, options, devtools) {
     options = { ...options };
-    const connection = new import_crConnection.CRConnection(transport, options.protocolLogger, options.browserLogsCollector);
+    const connection = new import_crConnection.CRConnection(parent, transport, options.protocolLogger, options.browserLogsCollector);
     const browser = new CRBrowser(parent, connection, options);
     browser._devtools = devtools;
     if (browser.isClank())
@@ -210,7 +210,7 @@ class CRBrowser extends import_browser.Browser {
   }
   _findOwningPage(frameId) {
     for (const crPage of this._crPages.values()) {
-      const frame = crPage._page._frameManager.frame(frameId);
+      const frame = crPage._page.frameManager.frame(frameId);
       if (frame)
         return crPage;
     }
@@ -243,7 +243,7 @@ class CRBrowser extends import_browser.Browser {
   }
   async startTracing(page, options = {}) {
     (0, import_assert.assert)(!this._tracingRecording, "Cannot start recording trace while already recording trace.");
-    this._tracingClient = page ? page._delegate._mainFrameSession._client : this._session;
+    this._tracingClient = page ? page.delegate._mainFrameSession._client : this._session;
     const defaultCategories = [
       "-*",
       "devtools.timeline",
@@ -322,26 +322,61 @@ class CRBrowserContext extends import_browserContext.BrowserContext {
   possiblyUninitializedPages() {
     return this._crPages().map((crPage) => crPage._page);
   }
-  async doCreateNewPage() {
-    (0, import_browserContext.assertBrowserContextIsNotOwned)(this);
+  async doCreateNewPage(markAsServerSideOnly) {
     const { targetId } = await this._browser._session.send("Target.createTarget", { url: "about:blank", browserContextId: this._browserContextId });
-    return this._browser._crPages.get(targetId)._page;
+    const page = this._browser._crPages.get(targetId)._page;
+    if (markAsServerSideOnly)
+      page.markAsServerSideOnly();
+    return page;
   }
   async doGetCookies(urls) {
     const { cookies } = await this._browser._session.send("Storage.getCookies", { browserContextId: this._browserContextId });
     return network.filterCookies(cookies.map((c) => {
-      const copy = { sameSite: "Lax", ...c };
-      delete copy.size;
-      delete copy.priority;
-      delete copy.session;
-      delete copy.sameParty;
-      delete copy.sourceScheme;
-      delete copy.sourcePort;
+      const { name, value, domain, path: path2, expires, httpOnly, secure, sameSite } = c;
+      const copy = {
+        name,
+        value,
+        domain,
+        path: path2,
+        expires,
+        httpOnly,
+        secure,
+        sameSite: sameSite ?? "Lax"
+      };
+      if (c.partitionKey) {
+        copy._crHasCrossSiteAncestor = c.partitionKey.hasCrossSiteAncestor;
+        copy.partitionKey = c.partitionKey.topLevelSite;
+      }
       return copy;
     }), urls);
   }
   async addCookies(cookies) {
-    await this._browser._session.send("Storage.setCookies", { cookies: network.rewriteCookies(cookies), browserContextId: this._browserContextId });
+    function toChromiumCookie(cookie) {
+      const { name, value, url, domain, path: path2, expires, httpOnly, secure, sameSite, partitionKey, _crHasCrossSiteAncestor } = cookie;
+      const copy = {
+        name,
+        value,
+        url,
+        domain,
+        path: path2,
+        expires,
+        httpOnly,
+        secure,
+        sameSite
+      };
+      if (partitionKey) {
+        copy.partitionKey = {
+          topLevelSite: partitionKey,
+          // _crHasCrossSiteAncestor is non-standard, set it true by default if the cookie is partitioned.
+          hasCrossSiteAncestor: _crHasCrossSiteAncestor ?? true
+        };
+      }
+      return copy;
+    }
+    await this._browser._session.send("Storage.setCookies", {
+      cookies: network.rewriteCookies(cookies).map(toChromiumCookie),
+      browserContextId: this._browserContextId
+    });
   }
   async doClearCookies() {
     await this._browser._session.send("Storage.clearCookies", { browserContextId: this._browserContextId });
@@ -363,7 +398,8 @@ class CRBrowserContext extends import_browserContext.BrowserContext {
       ["payment-handler", "paymentHandler"],
       // chrome-specific permissions we have.
       ["midi-sysex", "midiSysex"],
-      ["storage-access", "storageAccess"]
+      ["storage-access", "storageAccess"],
+      ["local-fonts", "localFonts"]
     ]);
     const filtered = permissions.map((permission) => {
       const protocolPermission = webPermissionToProtocol.get(permission);
@@ -380,55 +416,56 @@ class CRBrowserContext extends import_browserContext.BrowserContext {
     (0, import_browserContext.verifyGeolocation)(geolocation);
     this._options.geolocation = geolocation;
     for (const page of this.pages())
-      await page._delegate.updateGeolocation();
+      await page.delegate.updateGeolocation();
   }
-  async setExtraHTTPHeaders(headers) {
-    this._options.extraHTTPHeaders = headers;
+  async doUpdateExtraHTTPHeaders() {
     for (const page of this.pages())
-      await page._delegate.updateExtraHTTPHeaders();
+      await page.delegate.updateExtraHTTPHeaders();
     for (const sw of this.serviceWorkers())
       await sw.updateExtraHTTPHeaders();
   }
   async setUserAgent(userAgent) {
     this._options.userAgent = userAgent;
     for (const page of this.pages())
-      await page._delegate.updateUserAgent();
+      await page.delegate.updateUserAgent();
   }
-  async setOffline(offline) {
-    this._options.offline = offline;
+  async doUpdateOffline() {
     for (const page of this.pages())
-      await page._delegate.updateOffline();
+      await page.delegate.updateOffline();
     for (const sw of this.serviceWorkers())
       await sw.updateOffline();
   }
   async doSetHTTPCredentials(httpCredentials) {
     this._options.httpCredentials = httpCredentials;
     for (const page of this.pages())
-      await page._delegate.updateHttpCredentials();
+      await page.delegate.updateHttpCredentials();
     for (const sw of this.serviceWorkers())
       await sw.updateHttpCredentials();
   }
   async doAddInitScript(initScript) {
     for (const page of this.pages())
-      await page._delegate.addInitScript(initScript);
+      await page.delegate.addInitScript(initScript);
   }
-  async doRemoveNonInternalInitScripts() {
+  async doRemoveInitScripts(initScripts) {
     for (const page of this.pages())
-      await page._delegate.removeNonInternalInitScripts();
+      await page.delegate.removeInitScripts(initScripts);
   }
   async doUpdateRequestInterception() {
     for (const page of this.pages())
-      await page._delegate.updateRequestInterception();
+      await page.delegate.updateRequestInterception();
     for (const sw of this.serviceWorkers())
       await sw.updateRequestInterception();
   }
+  async doUpdateDefaultViewport() {
+  }
+  async doUpdateDefaultEmulatedMedia() {
+  }
+  async doExposePlaywrightBinding() {
+    for (const page of this._crPages())
+      await page.exposePlaywrightBinding();
+  }
   async doClose(reason) {
-    const openedBeforeUnloadDialogs = [];
-    for (const crPage of this._crPages()) {
-      const dialogs = [...crPage._page._frameManager._openedDialogs].filter((dialog) => dialog.type() === "beforeunload");
-      openedBeforeUnloadDialogs.push(...dialogs);
-    }
-    await Promise.all(openedBeforeUnloadDialogs.map((dialog) => dialog.dismiss()));
+    await this.dialogManager.closeBeforeUnloadDialogs();
     if (!this._browserContextId) {
       await this.stopVideoRecording();
       await this._browser.close({ reason });
@@ -437,7 +474,7 @@ class CRBrowserContext extends import_browserContext.BrowserContext {
     await this._browser._session.send("Target.disposeBrowserContext", { browserContextId: this._browserContextId });
     this._browser._contexts.delete(this._browserContextId);
     for (const [targetId, serviceWorker] of this._browser._serviceWorkers) {
-      if (serviceWorker._browserContext !== this)
+      if (serviceWorker.browserContext !== this)
         continue;
       serviceWorker.didClose();
       this._browser._serviceWorkers.delete(targetId);
@@ -473,14 +510,14 @@ class CRBrowserContext extends import_browserContext.BrowserContext {
     return result;
   }
   serviceWorkers() {
-    return Array.from(this._browser._serviceWorkers.values()).filter((serviceWorker) => serviceWorker._browserContext === this);
+    return Array.from(this._browser._serviceWorkers.values()).filter((serviceWorker) => serviceWorker.browserContext === this);
   }
   async newCDPSession(page) {
     let targetId = null;
     if (page instanceof import_page.Page) {
-      targetId = page._delegate._targetId;
+      targetId = page.delegate._targetId;
     } else if (page instanceof import_frames.Frame) {
-      const session = page._page._delegate._sessions.get(page._id);
+      const session = page._page.delegate._sessions.get(page._id);
       if (!session)
         throw new Error(`This frame does not have a separate CDP session, it is a part of the parent frame's session`);
       targetId = session._targetId;
