@@ -399,7 +399,8 @@ const DOWNLOAD_PATHS = {
     "win64": "builds/android/%s/android.zip"
   },
   // TODO(bidi): implement downloads.
-  "bidi": {}
+  "_bidiFirefox": {},
+  "_bidiChromium": {}
 };
 const registryDirectory = (() => {
   let result;
@@ -654,8 +655,8 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
     }));
     this._executables.push({
       type: "browser",
-      name: "bidi-chromium",
-      browserName: "bidi",
+      name: "_bidiChromium",
+      browserName: "_bidiChromium",
       directory: chromium.dir,
       executablePath: () => chromiumExecutable,
       executablePathOrDie: (sdkLanguage) => executablePathOrDie("chromium", chromiumExecutable, chromium.installByDefault, sdkLanguage),
@@ -775,20 +776,6 @@ ${(0, import_ascii.wrapInASCIIBox)(prettyMessage, 1)}`);
       _dependencyGroup: "tools",
       _isHermeticInstallation: true
     });
-    this._executables.push({
-      type: "browser",
-      name: "bidi",
-      browserName: "bidi",
-      directory: void 0,
-      executablePath: () => void 0,
-      executablePathOrDie: () => "",
-      installType: "none",
-      _validateHostRequirements: () => Promise.resolve(),
-      downloadURLs: [],
-      _install: () => Promise.resolve(),
-      _dependencyGroup: "tools",
-      _isHermeticInstallation: true
-    });
   }
   _createChromiumChannel(name, lookAt, install) {
     const executablePath = (sdkLanguage, shouldThrow) => {
@@ -859,7 +846,7 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
     return {
       type: "channel",
       name,
-      browserName: "bidi",
+      browserName: "_bidiFirefox",
       directory: void 0,
       executablePath: (sdkLanguage) => executablePath(sdkLanguage, false),
       executablePathOrDie: (sdkLanguage) => executablePath(sdkLanguage, true),
@@ -874,7 +861,7 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       const suffix = lookAt[process.platform];
       if (!suffix) {
         if (shouldThrow)
-          throw new Error(`Firefox distribution '${name}' is not supported on ${process.platform}`);
+          throw new Error(`Chromium distribution '${name}' is not supported on ${process.platform}`);
         return void 0;
       }
       const prefixes = process.platform === "win32" ? [
@@ -900,7 +887,7 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
     return {
       type: "channel",
       name,
-      browserName: "bidi",
+      browserName: "_bidiChromium",
       directory: void 0,
       executablePath: (sdkLanguage) => executablePath(sdkLanguage, false),
       executablePathOrDie: (sdkLanguage) => executablePath(sdkLanguage, true),
@@ -963,14 +950,15 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
       });
       await import_fs.default.promises.mkdir(linksDir, { recursive: true });
       await import_fs.default.promises.writeFile(import_path.default.join(linksDir, (0, import_utils.calculateSha1)(PACKAGE_PATH)), PACKAGE_PATH);
-      await this._validateInstallationCache(linksDir);
+      if (!(0, import_utils.getAsBooleanFromENV)("PLAYWRIGHT_SKIP_BROWSER_GC"))
+        await this._validateInstallationCache(linksDir);
       for (const executable of executables) {
         if (!executable._install)
           throw new Error(`ERROR: Playwright does not support installing ${executable.name}`);
         const { embedderName } = (0, import_userAgent.getEmbedderName)();
         if (!(0, import_utils.getAsBooleanFromENV)("CI") && !executable._isHermeticInstallation && !forceReinstall && executable.executablePath(embedderName)) {
           const command = buildPlaywrightCLICommand(embedderName, "install --force " + executable.name);
-          throw new Error("\n" + (0, import_ascii.wrapInASCIIBox)([
+          process.stderr.write("\n" + (0, import_ascii.wrapInASCIIBox)([
             `ATTENTION: "${executable.name}" is already installed on the system!`,
             ``,
             `"${executable.name}" installation is not hermetic; installing newer version`,
@@ -984,7 +972,8 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
             `    ${command}`,
             ``,
             `<3 Playwright Team`
-          ].join("\n"), 1));
+          ].join("\n"), 1) + "\n\n");
+          return;
         }
         await executable._install();
       }
@@ -1091,7 +1080,7 @@ Run "${buildPlaywrightCLICommand(sdkLanguage, "install " + name)}"` : "";
     const title = descriptor.browserVersion ? `${displayName} ${descriptor.browserVersion} (playwright build v${descriptor.revision})` : `${displayName} playwright build v${descriptor.revision}`;
     const downloadFileName = `playwright-download-${descriptor.name}-${import_hostPlatform.hostPlatform}-${descriptor.revision}.zip`;
     const downloadSocketTimeoutEnv = (0, import_utils.getFromENV)("PLAYWRIGHT_DOWNLOAD_CONNECTION_TIMEOUT");
-    const downloadSocketTimeout = +(downloadSocketTimeoutEnv || "0") || 3e4;
+    const downloadSocketTimeout = +(downloadSocketTimeoutEnv || "0") || import_network.NET_DEFAULT_TIMEOUT;
     await (0, import_browserFetcher.downloadBrowserWithProgressBar)(title, descriptor.dir, executablePath, downloadURLs, downloadFileName, downloadSocketTimeout).catch((e) => {
       throw new Error(`Failed to download ${title}, caused by
 ${e.stack}`);
@@ -1100,7 +1089,7 @@ ${e.stack}`);
   async _installMSEdgeChannel(channel, scripts) {
     const scriptArgs = [];
     if (process.platform !== "linux") {
-      const products = lowercaseAllKeys(JSON.parse(await (0, import_network.fetchData)({ url: "https://edgeupdates.microsoft.com/api/products" })));
+      const products = lowercaseAllKeys(JSON.parse(await (0, import_network.fetchData)(void 0, { url: "https://edgeupdates.microsoft.com/api/products" })));
       const productName = {
         "msedge": "Stable",
         "msedge-beta": "Beta",
@@ -1149,8 +1138,19 @@ ${e.stack}`);
         throw new Error(`Failed to install ${channel}`);
     }
   }
+  async listInstalledBrowsers() {
+    const linksDir = import_path.default.join(registryDirectory, ".links");
+    const { browsers } = await this._traverseBrowserInstallations(linksDir);
+    return browsers.filter((browser) => import_fs.default.existsSync(browser.browserPath));
+  }
   async _validateInstallationCache(linksDir) {
-    const usedBrowserPaths = /* @__PURE__ */ new Set();
+    const { browsers, brokenLinks } = await this._traverseBrowserInstallations(linksDir);
+    await this._deleteStaleBrowsers(browsers);
+    await this._deleteBrokenInstallations(brokenLinks);
+  }
+  async _traverseBrowserInstallations(linksDir) {
+    const browserList = [];
+    const brokenLinks = [];
     for (const fileName of await import_fs.default.promises.readdir(linksDir)) {
       const linkPath = import_path.default.join(linksDir, fileName);
       let linkTarget = "";
@@ -1162,28 +1162,43 @@ ${e.stack}`);
           const descriptor = descriptors.find((d) => d.name === browserName);
           if (!descriptor)
             continue;
-          const usedBrowserPath = descriptor.dir;
-          const browserRevision = parseInt(descriptor.revision, 10);
-          const shouldHaveMarkerFile = browserName === "chromium" && (browserRevision >= 786218 || browserRevision < 3e5) || browserName === "firefox" && browserRevision >= 1128 || browserName === "webkit" && browserRevision >= 1307 || // All new applications have a marker file right away.
-          browserName !== "firefox" && browserName !== "chromium" && browserName !== "webkit";
-          if (!shouldHaveMarkerFile || await (0, import_fileUtils.existsAsync)(browserDirectoryToMarkerFilePath(usedBrowserPath)))
-            usedBrowserPaths.add(usedBrowserPath);
+          const browserPath = descriptor.dir;
+          const browserVersion = parseInt(descriptor.revision, 10);
+          browserList.push({
+            browserName,
+            browserVersion,
+            browserPath,
+            referenceDir: linkTarget
+          });
         }
       } catch (e) {
-        await import_fs.default.promises.unlink(linkPath).catch((e2) => {
-        });
+        brokenLinks.push(linkPath);
       }
     }
-    if (!(0, import_utils.getAsBooleanFromENV)("PLAYWRIGHT_SKIP_BROWSER_GC")) {
-      let downloadedBrowsers = (await import_fs.default.promises.readdir(registryDirectory)).map((file) => import_path.default.join(registryDirectory, file));
-      downloadedBrowsers = downloadedBrowsers.filter((file) => isBrowserDirectory(file));
-      const directories = new Set(downloadedBrowsers);
-      for (const browserDirectory of usedBrowserPaths)
-        directories.delete(browserDirectory);
-      for (const directory of directories)
-        (0, import_browserFetcher.logPolitely)("Removing unused browser at " + directory);
-      await (0, import_fileUtils.removeFolders)([...directories]);
+    return { browsers: browserList, brokenLinks };
+  }
+  async _deleteStaleBrowsers(browserList) {
+    const usedBrowserPaths = /* @__PURE__ */ new Set();
+    for (const browser of browserList) {
+      const { browserName, browserVersion, browserPath } = browser;
+      const shouldHaveMarkerFile = browserName === "chromium" && (browserVersion >= 786218 || browserVersion < 3e5) || browserName === "firefox" && browserVersion >= 1128 || browserName === "webkit" && browserVersion >= 1307 || // All new applications have a marker file right away.
+      browserName !== "firefox" && browserName !== "chromium" && browserName !== "webkit";
+      if (!shouldHaveMarkerFile || await (0, import_fileUtils.existsAsync)(browserDirectoryToMarkerFilePath(browserPath)))
+        usedBrowserPaths.add(browserPath);
     }
+    let downloadedBrowsers = (await import_fs.default.promises.readdir(registryDirectory)).map((file) => import_path.default.join(registryDirectory, file));
+    downloadedBrowsers = downloadedBrowsers.filter((file) => isBrowserDirectory(file));
+    const directories = new Set(downloadedBrowsers);
+    for (const browserDirectory of usedBrowserPaths)
+      directories.delete(browserDirectory);
+    for (const directory of directories)
+      (0, import_browserFetcher.logPolitely)("Removing unused browser at " + directory);
+    await (0, import_fileUtils.removeFolders)([...directories]);
+  }
+  async _deleteBrokenInstallations(brokenLinks) {
+    for (const linkPath of brokenLinks)
+      await import_fs.default.promises.unlink(linkPath).catch((e) => {
+      });
   }
 }
 function browserDirectoryToMarkerFilePath(browserDirectory) {
